﻿using System;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 핫키 관리자
    /// </summary>
    internal class HotKeyManager
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 이용 가능 여부
        /// </summary>
        public static bool Enable = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 핫 키 여부 구하기 - IsHotkey(e, key, ctrl, shift, alt)

        /// <summary>
        /// 핫 키 여부 구하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <param name="key">키</param>
        /// <param name="ctrl">CTRL 키 눌림 여부</param>
        /// <param name="shift">SHIFT 키 눌림 여부</param>
        /// <param name="alt">ALT 키 눌림 여부</param>
        /// <returns>핫 키 여부</returns>
        public static bool IsHotkey(KeyEventArgs e, Keys key, bool ctrl = false, bool shift = false, bool alt = false)
        {
            return e.KeyCode == key && e.Control == ctrl && e.Shift == shift && e.Alt == alt;
        }

        #endregion
        #region 핫 키 추가하기 - AddHotKey(form, action, key, ctrl, shift, alt)

        /// <summary>
        /// 핫키 추가하기
        /// </summary>
        /// <param name="form">폼</param>
        /// <param name="action">액션</param>
        /// <param name="key">키</param>
        /// <param name="ctrl">CTRL 키 눌림 여부</param>
        /// <param name="shift">SHIFT 키 눌림 여부</param>
        /// <param name="alt">ALT 키 눌림 여부</param>
        public static void AddHotKey(Form form, Action action, Keys key, bool ctrl = false, bool shift = false, bool alt = false)
        {
            form.KeyPreview = true;

            form.KeyDown += delegate(object sender, KeyEventArgs e)
            {
                if(IsHotkey(e, key, ctrl, shift, alt))
                {
                    action();
                }
            };
        }

        #endregion
    }
}