﻿using System.Windows.Forms;

using ScintillaNET;

namespace TestProject
{
    /// <summary>
    /// 검색 관리자
    /// </summary>
    internal class SearchManager
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 에디터
        /// </summary>
        public static Scintilla Editor;

        /// <summary>
        /// 검색 텍스트 박스
        /// </summary>
        public static TextBox SearchTextBox;

        /// <summary>
        /// 마지막 검색 문자열
        /// </summary>
        public static string LastSearchString = string.Empty;

        /// <summary>
        /// 마지막 검색 인덱스
        /// </summary>
        public static int LastSearchIndex;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 찾기 - Find(next, incremental)

        /// <summary>
        /// 찾기
        /// </summary>
        /// <param name="next">다음 찾기 여부</param>
        /// <param name="incremental">증분 검색 여부</param>
        public static void Find(bool next, bool incremental)
        {
            LastSearchString = SearchTextBox.Text;

            if(LastSearchString.Length > 0)
            {
                if(next)
                {
                    Editor.TargetStart = LastSearchIndex - 1;
                    Editor.TargetEnd   = LastSearchIndex + (LastSearchString.Length + 1);
                    Editor.SearchFlags = SearchFlags.None;

                    if(!incremental || Editor.SearchInTarget(LastSearchString) == -1)
                    {
                        Editor.TargetStart = Editor.CurrentPosition;
                        Editor.TargetEnd   = Editor.TextLength;
                        Editor.SearchFlags = SearchFlags.None;

                        if(Editor.SearchInTarget(LastSearchString) == -1)
                        {
                            Editor.TargetStart = 0;
                            Editor.TargetEnd   = Editor.TextLength;

                            if(Editor.SearchInTarget(LastSearchString) == -1)
                            {
                                Editor.ClearSelections();

                                return;
                            }
                        }
                    }
                }
                else
                {
                    Editor.TargetStart = 0;
                    Editor.TargetEnd   = Editor.CurrentPosition;
                    Editor.SearchFlags = SearchFlags.None;

                    if(Editor.SearchInTarget(LastSearchString) == -1)
                    {
                        Editor.TargetStart = Editor.CurrentPosition;
                        Editor.TargetEnd   = Editor.TextLength;

                        if(Editor.SearchInTarget(LastSearchString) == -1)
                        {
                            Editor.ClearSelections();

                            return;
                        }
                    }
                }

                LastSearchIndex = Editor.TargetStart;

                Editor.SetSelection(Editor.TargetEnd, Editor.TargetStart);

                Editor.ScrollCaret();
            }

            SearchTextBox.Focus();
        }

        #endregion
    }
}